# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'D:\a\1\s\fmpy\gui\forms\AboutDialog.ui'
#
# Created by: PyQt5 UI code generator 5.12.3
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(474, 234)
        palette = QtGui.QPalette()
        brush = QtGui.QBrush(QtGui.QColor(255, 255, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Base, brush)
        brush = QtGui.QBrush(QtGui.QColor(255, 255, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Window, brush)
        brush = QtGui.QBrush(QtGui.QColor(255, 255, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Base, brush)
        brush = QtGui.QBrush(QtGui.QColor(255, 255, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Window, brush)
        brush = QtGui.QBrush(QtGui.QColor(255, 255, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Base, brush)
        brush = QtGui.QBrush(QtGui.QColor(255, 255, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Window, brush)
        Dialog.setPalette(palette)
        Dialog.setModal(True)
        self.formLayout = QtWidgets.QFormLayout(Dialog)
        self.formLayout.setContentsMargins(50, 50, 50, 9)
        self.formLayout.setObjectName("formLayout")
        self.label = QtWidgets.QLabel(Dialog)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label.setFont(font)
        self.label.setObjectName("label")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label)
        self.fmpyVersionLabel = QtWidgets.QLabel(Dialog)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.fmpyVersionLabel.setFont(font)
        self.fmpyVersionLabel.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByMouse)
        self.fmpyVersionLabel.setObjectName("fmpyVersionLabel")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.fmpyVersionLabel)
        self.label_10 = QtWidgets.QLabel(Dialog)
        self.label_10.setObjectName("label_10")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label_10)
        self.fmiPlatformLabel = QtWidgets.QLabel(Dialog)
        self.fmiPlatformLabel.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByMouse)
        self.fmiPlatformLabel.setObjectName("fmiPlatformLabel")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.fmiPlatformLabel)
        self.label_6 = QtWidgets.QLabel(Dialog)
        self.label_6.setObjectName("label_6")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.label_6)
        self.installationPathLabel = QtWidgets.QLabel(Dialog)
        self.installationPathLabel.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByMouse)
        self.installationPathLabel.setObjectName("installationPathLabel")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.installationPathLabel)
        self.label_4 = QtWidgets.QLabel(Dialog)
        self.label_4.setObjectName("label_4")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.LabelRole, self.label_4)
        self.pythonInterpreterLabel = QtWidgets.QLabel(Dialog)
        self.pythonInterpreterLabel.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByMouse)
        self.pythonInterpreterLabel.setObjectName("pythonInterpreterLabel")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.FieldRole, self.pythonInterpreterLabel)
        self.label_2 = QtWidgets.QLabel(Dialog)
        self.label_2.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.label_2.setObjectName("label_2")
        self.formLayout.setWidget(6, QtWidgets.QFormLayout.LabelRole, self.label_2)
        self.pythonVersionLabel = QtWidgets.QLabel(Dialog)
        self.pythonVersionLabel.setWordWrap(True)
        self.pythonVersionLabel.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByMouse)
        self.pythonVersionLabel.setObjectName("pythonVersionLabel")
        self.formLayout.setWidget(6, QtWidgets.QFormLayout.FieldRole, self.pythonVersionLabel)
        spacerItem = QtWidgets.QSpacerItem(20, 60, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.formLayout.setItem(8, QtWidgets.QFormLayout.FieldRole, spacerItem)
        self.label_8 = QtWidgets.QLabel(Dialog)
        self.label_8.setObjectName("label_8")
        self.formLayout.setWidget(9, QtWidgets.QFormLayout.FieldRole, self.label_8)

        self.retranslateUi(Dialog)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "About FMPy"))
        self.label.setText(_translate("Dialog", "FMPy Version"))
        self.fmpyVersionLabel.setText(_translate("Dialog", "2.0.0"))
        self.label_10.setText(_translate("Dialog", "FMI Platform"))
        self.fmiPlatformLabel.setText(_translate("Dialog", "win64"))
        self.label_6.setText(_translate("Dialog", "Installation Path"))
        self.installationPathLabel.setText(_translate("Dialog", "C:\\Anaconda3\\lib\\site-packages\\fmpy"))
        self.label_4.setText(_translate("Dialog", "Python Interpreter"))
        self.pythonInterpreterLabel.setText(_translate("Dialog", "C:\\Anaconda3\\python.exe"))
        self.label_2.setText(_translate("Dialog", "Python Version"))
        self.pythonVersionLabel.setText(_translate("Dialog", "3.5.2 |Anaconda custom (64-bit)| (default, Jul  5 2016, 11:41:13) [MSC v.1900 64 bit (AMD64)]"))
        self.label_8.setText(_translate("Dialog", "© 2020 Dassault Systèmes. All rights reserved."))
