# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'D:\a\1\s\fmpy\gui\forms\MainWindow.ui'
#
# Created by: PyQt5 UI code generator 5.12.3
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(1125, 836)
        palette = QtGui.QPalette()
        brush = QtGui.QBrush(QtGui.QColor(255, 255, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Base, brush)
        brush = QtGui.QBrush(QtGui.QColor(255, 255, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Window, brush)
        brush = QtGui.QBrush(QtGui.QColor(255, 255, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Base, brush)
        brush = QtGui.QBrush(QtGui.QColor(255, 255, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Window, brush)
        brush = QtGui.QBrush(QtGui.QColor(255, 255, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Base, brush)
        brush = QtGui.QBrush(QtGui.QColor(255, 255, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Window, brush)
        MainWindow.setPalette(palette)
        MainWindow.setAcceptDrops(True)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/icons/app_icon.ico"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        MainWindow.setWindowIcon(icon)
        MainWindow.setUnifiedTitleAndToolBarOnMac(True)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout_2.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.stackedWidget = QtWidgets.QStackedWidget(self.centralwidget)
        self.stackedWidget.setObjectName("stackedWidget")
        self.startPage = QtWidgets.QWidget()
        self.startPage.setObjectName("startPage")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.startPage)
        self.verticalLayout_2.setSpacing(15)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        spacerItem = QtWidgets.QSpacerItem(20, 113, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem)
        self.widget_8 = QtWidgets.QWidget(self.startPage)
        self.widget_8.setObjectName("widget_8")
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout(self.widget_8)
        self.horizontalLayout_7.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_7.addItem(spacerItem1)
        self.label_9 = QtWidgets.QLabel(self.widget_8)
        self.label_9.setPixmap(QtGui.QPixmap(":/icons/start-background.png"))
        self.label_9.setObjectName("label_9")
        self.horizontalLayout_7.addWidget(self.label_9)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_7.addItem(spacerItem2)
        self.verticalLayout_2.addWidget(self.widget_8)
        self.widget_9 = QtWidgets.QWidget(self.startPage)
        self.widget_9.setObjectName("widget_9")
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout(self.widget_9)
        self.horizontalLayout_8.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_8.addItem(spacerItem3)
        self.label_11 = QtWidgets.QLabel(self.widget_9)
        self.label_11.setObjectName("label_11")
        self.horizontalLayout_8.addWidget(self.label_11)
        self.openButton = QtWidgets.QPushButton(self.widget_9)
        self.openButton.setObjectName("openButton")
        self.horizontalLayout_8.addWidget(self.openButton)
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_8.addItem(spacerItem4)
        self.verticalLayout_2.addWidget(self.widget_9)
        self.widget = QtWidgets.QWidget(self.startPage)
        self.widget.setMinimumSize(QtCore.QSize(0, 20))
        self.widget.setObjectName("widget")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.widget)
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem5 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem5)
        self.recentFilesGroupBox = QtWidgets.QGroupBox(self.widget)
        self.recentFilesGroupBox.setMinimumSize(QtCore.QSize(200, 0))
        self.recentFilesGroupBox.setFlat(True)
        self.recentFilesGroupBox.setObjectName("recentFilesGroupBox")
        self.horizontalLayout.addWidget(self.recentFilesGroupBox)
        spacerItem6 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem6)
        self.verticalLayout_2.addWidget(self.widget)
        spacerItem7 = QtWidgets.QSpacerItem(20, 112, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem7)
        self.stackedWidget.addWidget(self.startPage)
        self.settingsPage = QtWidgets.QWidget()
        self.settingsPage.setObjectName("settingsPage")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.settingsPage)
        self.verticalLayout.setContentsMargins(15, 15, 15, 15)
        self.verticalLayout.setObjectName("verticalLayout")
        self.infoGroupBox = QtWidgets.QGroupBox(self.settingsPage)
        self.infoGroupBox.setFlat(True)
        self.infoGroupBox.setObjectName("infoGroupBox")
        self.gridLayout_9 = QtWidgets.QGridLayout(self.infoGroupBox)
        self.gridLayout_9.setContentsMargins(-1, 12, -1, -1)
        self.gridLayout_9.setHorizontalSpacing(9)
        self.gridLayout_9.setVerticalSpacing(12)
        self.gridLayout_9.setObjectName("gridLayout_9")
        self.modelNameLabel = QtWidgets.QLabel(self.infoGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.modelNameLabel.sizePolicy().hasHeightForWidth())
        self.modelNameLabel.setSizePolicy(sizePolicy)
        self.modelNameLabel.setText("")
        self.modelNameLabel.setWordWrap(True)
        self.modelNameLabel.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByMouse)
        self.modelNameLabel.setObjectName("modelNameLabel")
        self.gridLayout_9.addWidget(self.modelNameLabel, 2, 1, 1, 1)
        self.label_3 = QtWidgets.QLabel(self.infoGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_3.sizePolicy().hasHeightForWidth())
        self.label_3.setSizePolicy(sizePolicy)
        self.label_3.setObjectName("label_3")
        self.gridLayout_9.addWidget(self.label_3, 0, 0, 1, 1)
        self.label_20 = QtWidgets.QLabel(self.infoGroupBox)
        self.label_20.setObjectName("label_20")
        self.gridLayout_9.addWidget(self.label_20, 6, 0, 1, 1)
        self.fmiVersionLabel = QtWidgets.QLabel(self.infoGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.fmiVersionLabel.sizePolicy().hasHeightForWidth())
        self.fmiVersionLabel.setSizePolicy(sizePolicy)
        self.fmiVersionLabel.setText("")
        self.fmiVersionLabel.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByMouse)
        self.fmiVersionLabel.setObjectName("fmiVersionLabel")
        self.gridLayout_9.addWidget(self.fmiVersionLabel, 0, 1, 1, 1)
        self.numberOfVariablesLabel = QtWidgets.QLabel(self.infoGroupBox)
        self.numberOfVariablesLabel.setText("")
        self.numberOfVariablesLabel.setObjectName("numberOfVariablesLabel")
        self.gridLayout_9.addWidget(self.numberOfVariablesLabel, 6, 1, 1, 1)
        self.fmiTypeLabel = QtWidgets.QLabel(self.infoGroupBox)
        self.fmiTypeLabel.setText("")
        self.fmiTypeLabel.setObjectName("fmiTypeLabel")
        self.gridLayout_9.addWidget(self.fmiTypeLabel, 1, 1, 1, 1)
        self.label_8 = QtWidgets.QLabel(self.infoGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_8.sizePolicy().hasHeightForWidth())
        self.label_8.setSizePolicy(sizePolicy)
        self.label_8.setObjectName("label_8")
        self.gridLayout_9.addWidget(self.label_8, 7, 0, 1, 1)
        self.numberOfEventIndicatorsLabel = QtWidgets.QLabel(self.infoGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.numberOfEventIndicatorsLabel.sizePolicy().hasHeightForWidth())
        self.numberOfEventIndicatorsLabel.setSizePolicy(sizePolicy)
        self.numberOfEventIndicatorsLabel.setText("")
        self.numberOfEventIndicatorsLabel.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByMouse)
        self.numberOfEventIndicatorsLabel.setObjectName("numberOfEventIndicatorsLabel")
        self.gridLayout_9.addWidget(self.numberOfEventIndicatorsLabel, 5, 1, 1, 1)
        self.label_12 = QtWidgets.QLabel(self.infoGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_12.sizePolicy().hasHeightForWidth())
        self.label_12.setSizePolicy(sizePolicy)
        self.label_12.setObjectName("label_12")
        self.gridLayout_9.addWidget(self.label_12, 5, 0, 1, 1)
        self.label_10 = QtWidgets.QLabel(self.infoGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_10.sizePolicy().hasHeightForWidth())
        self.label_10.setSizePolicy(sizePolicy)
        self.label_10.setObjectName("label_10")
        self.gridLayout_9.addWidget(self.label_10, 4, 0, 1, 1)
        self.label_4 = QtWidgets.QLabel(self.infoGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_4.sizePolicy().hasHeightForWidth())
        self.label_4.setSizePolicy(sizePolicy)
        self.label_4.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.label_4.setObjectName("label_4")
        self.gridLayout_9.addWidget(self.label_4, 2, 0, 1, 1)
        self.platformsLabel = QtWidgets.QLabel(self.infoGroupBox)
        self.platformsLabel.setText("")
        self.platformsLabel.setObjectName("platformsLabel")
        self.gridLayout_9.addWidget(self.platformsLabel, 3, 1, 1, 1)
        self.generationDateAndTimeLabel = QtWidgets.QLabel(self.infoGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.generationDateAndTimeLabel.sizePolicy().hasHeightForWidth())
        self.generationDateAndTimeLabel.setSizePolicy(sizePolicy)
        self.generationDateAndTimeLabel.setText("")
        self.generationDateAndTimeLabel.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByMouse)
        self.generationDateAndTimeLabel.setObjectName("generationDateAndTimeLabel")
        self.gridLayout_9.addWidget(self.generationDateAndTimeLabel, 7, 1, 1, 1)
        self.label_5 = QtWidgets.QLabel(self.infoGroupBox)
        self.label_5.setObjectName("label_5")
        self.gridLayout_9.addWidget(self.label_5, 1, 0, 1, 1)
        self.label = QtWidgets.QLabel(self.infoGroupBox)
        self.label.setObjectName("label")
        self.gridLayout_9.addWidget(self.label, 3, 0, 1, 1)
        self.numberOfContinuousStatesLabel = QtWidgets.QLabel(self.infoGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.numberOfContinuousStatesLabel.sizePolicy().hasHeightForWidth())
        self.numberOfContinuousStatesLabel.setSizePolicy(sizePolicy)
        self.numberOfContinuousStatesLabel.setText("")
        self.numberOfContinuousStatesLabel.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByMouse)
        self.numberOfContinuousStatesLabel.setObjectName("numberOfContinuousStatesLabel")
        self.gridLayout_9.addWidget(self.numberOfContinuousStatesLabel, 4, 1, 1, 1)
        self.descriptionLabel = QtWidgets.QLabel(self.infoGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.descriptionLabel.sizePolicy().hasHeightForWidth())
        self.descriptionLabel.setSizePolicy(sizePolicy)
        self.descriptionLabel.setText("")
        self.descriptionLabel.setWordWrap(True)
        self.descriptionLabel.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByMouse)
        self.descriptionLabel.setObjectName("descriptionLabel")
        self.gridLayout_9.addWidget(self.descriptionLabel, 9, 1, 1, 2)
        self.label_22 = QtWidgets.QLabel(self.infoGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_22.sizePolicy().hasHeightForWidth())
        self.label_22.setSizePolicy(sizePolicy)
        self.label_22.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.label_22.setObjectName("label_22")
        self.gridLayout_9.addWidget(self.label_22, 9, 0, 1, 1)
        self.generationToolLabel = QtWidgets.QLabel(self.infoGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.generationToolLabel.sizePolicy().hasHeightForWidth())
        self.generationToolLabel.setSizePolicy(sizePolicy)
        self.generationToolLabel.setText("")
        self.generationToolLabel.setWordWrap(True)
        self.generationToolLabel.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByMouse)
        self.generationToolLabel.setObjectName("generationToolLabel")
        self.gridLayout_9.addWidget(self.generationToolLabel, 8, 1, 1, 2)
        self.label_7 = QtWidgets.QLabel(self.infoGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_7.sizePolicy().hasHeightForWidth())
        self.label_7.setSizePolicy(sizePolicy)
        self.label_7.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.label_7.setObjectName("label_7")
        self.gridLayout_9.addWidget(self.label_7, 8, 0, 1, 1)
        self.modelImageLabel = QtWidgets.QLabel(self.infoGroupBox)
        self.modelImageLabel.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.modelImageLabel.setObjectName("modelImageLabel")
        self.gridLayout_9.addWidget(self.modelImageLabel, 0, 2, 8, 1)
        self.verticalLayout.addWidget(self.infoGroupBox)
        self.settingsGroupBox = QtWidgets.QGroupBox(self.settingsPage)
        self.settingsGroupBox.setFlat(True)
        self.settingsGroupBox.setObjectName("settingsGroupBox")
        self.gridLayout_13 = QtWidgets.QGridLayout(self.settingsGroupBox)
        self.gridLayout_13.setObjectName("gridLayout_13")
        self.label_14 = QtWidgets.QLabel(self.settingsGroupBox)
        self.label_14.setObjectName("label_14")
        self.gridLayout_13.addWidget(self.label_14, 2, 0, 1, 1)
        self.applyDefaultStartValuesCheckBox = QtWidgets.QCheckBox(self.settingsGroupBox)
        self.applyDefaultStartValuesCheckBox.setObjectName("applyDefaultStartValuesCheckBox")
        self.gridLayout_13.addWidget(self.applyDefaultStartValuesCheckBox, 6, 0, 1, 2)
        self.inputCheckBox = QtWidgets.QCheckBox(self.settingsGroupBox)
        self.inputCheckBox.setObjectName("inputCheckBox")
        self.gridLayout_13.addWidget(self.inputCheckBox, 5, 0, 1, 1)
        self.label_2 = QtWidgets.QLabel(self.settingsGroupBox)
        self.label_2.setObjectName("label_2")
        self.gridLayout_13.addWidget(self.label_2, 1, 0, 1, 1)
        self.selectInputButton = QtWidgets.QToolButton(self.settingsGroupBox)
        self.selectInputButton.setEnabled(False)
        self.selectInputButton.setMinimumSize(QtCore.QSize(45, 22))
        self.selectInputButton.setPopupMode(QtWidgets.QToolButton.MenuButtonPopup)
        self.selectInputButton.setObjectName("selectInputButton")
        self.gridLayout_13.addWidget(self.selectInputButton, 5, 2, 1, 1)
        self.logFMICallsCheckBox = QtWidgets.QCheckBox(self.settingsGroupBox)
        self.logFMICallsCheckBox.setObjectName("logFMICallsCheckBox")
        self.gridLayout_13.addWidget(self.logFMICallsCheckBox, 7, 0, 1, 2)
        self.debugLoggingCheckBox = QtWidgets.QCheckBox(self.settingsGroupBox)
        self.debugLoggingCheckBox.setObjectName("debugLoggingCheckBox")
        self.gridLayout_13.addWidget(self.debugLoggingCheckBox, 8, 0, 1, 2)
        self.label_13 = QtWidgets.QLabel(self.settingsGroupBox)
        self.label_13.setObjectName("label_13")
        self.gridLayout_13.addWidget(self.label_13, 0, 0, 1, 1)
        self.inputFilenameLineEdit = QtWidgets.QLineEdit(self.settingsGroupBox)
        self.inputFilenameLineEdit.setEnabled(False)
        self.inputFilenameLineEdit.setObjectName("inputFilenameLineEdit")
        self.gridLayout_13.addWidget(self.inputFilenameLineEdit, 5, 1, 1, 1)
        self.widget_3 = QtWidgets.QWidget(self.settingsGroupBox)
        self.widget_3.setObjectName("widget_3")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.widget_3)
        self.horizontalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.stepSizeLineEdit = QtWidgets.QLineEdit(self.widget_3)
        self.stepSizeLineEdit.setMinimumSize(QtCore.QSize(70, 0))
        self.stepSizeLineEdit.setMaximumSize(QtCore.QSize(70, 16777215))
        self.stepSizeLineEdit.setObjectName("stepSizeLineEdit")
        self.horizontalLayout_2.addWidget(self.stepSizeLineEdit)
        spacerItem8 = QtWidgets.QSpacerItem(336, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem8)
        self.gridLayout_13.addWidget(self.widget_3, 1, 1, 1, 1)
        self.widget_10 = QtWidgets.QWidget(self.settingsGroupBox)
        self.widget_10.setObjectName("widget_10")
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout(self.widget_10)
        self.horizontalLayout_9.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        self.solverComboBox = QtWidgets.QComboBox(self.widget_10)
        self.solverComboBox.setMinimumSize(QtCore.QSize(90, 0))
        self.solverComboBox.setSizeAdjustPolicy(QtWidgets.QComboBox.AdjustToContents)
        self.solverComboBox.setObjectName("solverComboBox")
        self.solverComboBox.addItem("")
        self.solverComboBox.addItem("")
        self.horizontalLayout_9.addWidget(self.solverComboBox)
        spacerItem9 = QtWidgets.QSpacerItem(340, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_9.addItem(spacerItem9)
        self.gridLayout_13.addWidget(self.widget_10, 0, 1, 1, 1)
        self.widget_11 = QtWidgets.QWidget(self.settingsGroupBox)
        self.widget_11.setObjectName("widget_11")
        self.horizontalLayout_10 = QtWidgets.QHBoxLayout(self.widget_11)
        self.horizontalLayout_10.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_10.setObjectName("horizontalLayout_10")
        self.relativeToleranceLineEdit = QtWidgets.QLineEdit(self.widget_11)
        self.relativeToleranceLineEdit.setMinimumSize(QtCore.QSize(70, 0))
        self.relativeToleranceLineEdit.setMaximumSize(QtCore.QSize(70, 16777215))
        self.relativeToleranceLineEdit.setObjectName("relativeToleranceLineEdit")
        self.horizontalLayout_10.addWidget(self.relativeToleranceLineEdit)
        spacerItem10 = QtWidgets.QSpacerItem(359, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_10.addItem(spacerItem10)
        self.gridLayout_13.addWidget(self.widget_11, 2, 1, 1, 1)
        self.widget_5 = QtWidgets.QWidget(self.settingsGroupBox)
        self.widget_5.setObjectName("widget_5")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout(self.widget_5)
        self.horizontalLayout_4.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.widget_7 = QtWidgets.QWidget(self.widget_5)
        self.widget_7.setObjectName("widget_7")
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout(self.widget_7)
        self.horizontalLayout_6.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.maxSamplesLineEdit = QtWidgets.QLineEdit(self.widget_7)
        self.maxSamplesLineEdit.setMinimumSize(QtCore.QSize(70, 0))
        self.maxSamplesLineEdit.setMaximumSize(QtCore.QSize(70, 16777215))
        self.maxSamplesLineEdit.setObjectName("maxSamplesLineEdit")
        self.horizontalLayout_6.addWidget(self.maxSamplesLineEdit)
        spacerItem11 = QtWidgets.QSpacerItem(334, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_6.addItem(spacerItem11)
        self.horizontalLayout_4.addWidget(self.widget_7)
        self.gridLayout_13.addWidget(self.widget_5, 4, 1, 1, 1)
        self.outputIntervalRadioButton = QtWidgets.QRadioButton(self.settingsGroupBox)
        self.outputIntervalRadioButton.setChecked(False)
        self.outputIntervalRadioButton.setObjectName("outputIntervalRadioButton")
        self.gridLayout_13.addWidget(self.outputIntervalRadioButton, 3, 0, 1, 1)
        self.widget_2 = QtWidgets.QWidget(self.settingsGroupBox)
        self.widget_2.setObjectName("widget_2")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(self.widget_2)
        self.horizontalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.outputIntervalLineEdit = QtWidgets.QLineEdit(self.widget_2)
        self.outputIntervalLineEdit.setEnabled(False)
        self.outputIntervalLineEdit.setMinimumSize(QtCore.QSize(70, 0))
        self.outputIntervalLineEdit.setMaximumSize(QtCore.QSize(70, 16777215))
        self.outputIntervalLineEdit.setObjectName("outputIntervalLineEdit")
        self.horizontalLayout_3.addWidget(self.outputIntervalLineEdit)
        spacerItem12 = QtWidgets.QSpacerItem(558, 18, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem12)
        self.gridLayout_13.addWidget(self.widget_2, 3, 1, 1, 1)
        self.maxSamplesRadioButton = QtWidgets.QRadioButton(self.settingsGroupBox)
        self.maxSamplesRadioButton.setChecked(True)
        self.maxSamplesRadioButton.setObjectName("maxSamplesRadioButton")
        self.gridLayout_13.addWidget(self.maxSamplesRadioButton, 4, 0, 1, 1)
        self.verticalLayout.addWidget(self.settingsGroupBox)
        self.portsGroupBox = QtWidgets.QGroupBox(self.settingsPage)
        self.portsGroupBox.setFlat(True)
        self.portsGroupBox.setObjectName("portsGroupBox")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.portsGroupBox)
        self.verticalLayout_4.setContentsMargins(0, 5, 0, 0)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.graphicsView = QtWidgets.QGraphicsView(self.portsGroupBox)
        self.graphicsView.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.graphicsView.setObjectName("graphicsView")
        self.verticalLayout_4.addWidget(self.graphicsView)
        self.verticalLayout.addWidget(self.portsGroupBox)
        self.stackedWidget.addWidget(self.settingsPage)
        self.logPage = QtWidgets.QWidget()
        self.logPage.setObjectName("logPage")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.logPage)
        self.verticalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_3.setSpacing(0)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.logWidget = QtWidgets.QWidget(self.logPage)
        self.logWidget.setStyleSheet("QToolButton {\n"
"     padding: 1px;\n"
"     border: 1px solid transparent;\n"
"     border-radius: 2px;\n"
"     background: transparent;\n"
"    height: 14px;\n"
"}\n"
"\n"
"QToolButton:hover {\n"
"        border: 1px solid  rgba( 92, 163, 255, 20% );\n"
"        background-color: rgba( 92, 163, 255, 10% );\n"
"}\n"
"\n"
"QToolButton:checked, QToolButton:hover:pressed {\n"
"        border: 1px solid  rgba( 92, 163, 255, 50% );\n"
"        background-color: rgba( 92, 163, 255, 25% );\n"
"}")
        self.logWidget.setObjectName("logWidget")
        self.horizontalLayout_11 = QtWidgets.QHBoxLayout(self.logWidget)
        self.horizontalLayout_11.setContentsMargins(0, 4, 4, 6)
        self.horizontalLayout_11.setSpacing(4)
        self.horizontalLayout_11.setObjectName("horizontalLayout_11")
        self.clearLogButton = QtWidgets.QToolButton(self.logWidget)
        self.clearLogButton.setMinimumSize(QtCore.QSize(0, 21))
        self.clearLogButton.setAutoRaise(True)
        self.clearLogButton.setObjectName("clearLogButton")
        self.horizontalLayout_11.addWidget(self.clearLogButton)
        self.clearLogOnStartButton = QtWidgets.QToolButton(self.logWidget)
        self.clearLogOnStartButton.setMinimumSize(QtCore.QSize(0, 21))
        self.clearLogOnStartButton.setCheckable(True)
        self.clearLogOnStartButton.setChecked(True)
        self.clearLogOnStartButton.setAutoRaise(True)
        self.clearLogOnStartButton.setObjectName("clearLogOnStartButton")
        self.horizontalLayout_11.addWidget(self.clearLogOnStartButton)
        self.logFilterLineEdit = QtWidgets.QLineEdit(self.logWidget)
        self.logFilterLineEdit.setStyleSheet("QLineEdit {\n"
"     border: 1px solid #ddd;\n"
"     border-radius: 2px;\n"
"     padding: 1px 5px 2px 5px;\n"
"     background: transparent;\n"
"    font-size: 11px;\n"
" }")
        self.logFilterLineEdit.setClearButtonEnabled(True)
        self.logFilterLineEdit.setObjectName("logFilterLineEdit")
        self.horizontalLayout_11.addWidget(self.logFilterLineEdit)
        self.showDebugMessagesButton = QtWidgets.QToolButton(self.logWidget)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/icons/debug-16x16.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.showDebugMessagesButton.setIcon(icon1)
        self.showDebugMessagesButton.setCheckable(True)
        self.showDebugMessagesButton.setChecked(False)
        self.showDebugMessagesButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextBesideIcon)
        self.showDebugMessagesButton.setAutoRaise(True)
        self.showDebugMessagesButton.setObjectName("showDebugMessagesButton")
        self.horizontalLayout_11.addWidget(self.showDebugMessagesButton)
        self.showInfoMessagesButton = QtWidgets.QToolButton(self.logWidget)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/icons/info-16x16.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.showInfoMessagesButton.setIcon(icon2)
        self.showInfoMessagesButton.setCheckable(True)
        self.showInfoMessagesButton.setChecked(True)
        self.showInfoMessagesButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextBesideIcon)
        self.showInfoMessagesButton.setAutoRaise(True)
        self.showInfoMessagesButton.setObjectName("showInfoMessagesButton")
        self.horizontalLayout_11.addWidget(self.showInfoMessagesButton)
        self.showWarningMessagesButton = QtWidgets.QToolButton(self.logWidget)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/icons/warning-16x16.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.showWarningMessagesButton.setIcon(icon3)
        self.showWarningMessagesButton.setCheckable(True)
        self.showWarningMessagesButton.setChecked(True)
        self.showWarningMessagesButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextBesideIcon)
        self.showWarningMessagesButton.setAutoRaise(True)
        self.showWarningMessagesButton.setObjectName("showWarningMessagesButton")
        self.horizontalLayout_11.addWidget(self.showWarningMessagesButton)
        self.showErrorMessagesButton = QtWidgets.QToolButton(self.logWidget)
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/icons/error-16x16.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.showErrorMessagesButton.setIcon(icon4)
        self.showErrorMessagesButton.setCheckable(True)
        self.showErrorMessagesButton.setChecked(True)
        self.showErrorMessagesButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextBesideIcon)
        self.showErrorMessagesButton.setAutoRaise(True)
        self.showErrorMessagesButton.setObjectName("showErrorMessagesButton")
        self.horizontalLayout_11.addWidget(self.showErrorMessagesButton)
        self.verticalLayout_3.addWidget(self.logWidget)
        self.logTreeView = QtWidgets.QTreeView(self.logPage)
        self.logTreeView.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.logTreeView.setRootIsDecorated(False)
        self.logTreeView.setObjectName("logTreeView")
        self.logTreeView.header().setVisible(False)
        self.verticalLayout_3.addWidget(self.logTreeView)
        self.stackedWidget.addWidget(self.logPage)
        self.resultPage = QtWidgets.QWidget()
        self.resultPage.setObjectName("resultPage")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.resultPage)
        self.gridLayout_3.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.plotWidget = GraphicsLayoutWidget(self.resultPage)
        self.plotWidget.setObjectName("plotWidget")
        self.gridLayout_3.addWidget(self.plotWidget, 0, 0, 1, 1)
        self.stackedWidget.addWidget(self.resultPage)
        self.gridLayout_2.addWidget(self.stackedWidget, 0, 0, 1, 1)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 1125, 26))
        self.menubar.setObjectName("menubar")
        self.menu_File = QtWidgets.QMenu(self.menubar)
        self.menu_File.setObjectName("menu_File")
        self.menu_Help = QtWidgets.QMenu(self.menubar)
        self.menu_Help.setObjectName("menu_Help")
        self.menuFMI_Specifications = QtWidgets.QMenu(self.menu_Help)
        self.menuFMI_Specifications.setObjectName("menuFMI_Specifications")
        self.menu_Tools = QtWidgets.QMenu(self.menubar)
        self.menu_Tools.setObjectName("menu_Tools")
        MainWindow.setMenuBar(self.menubar)
        self.statusBar = QtWidgets.QStatusBar(MainWindow)
        self.statusBar.setStyleSheet("QStatusBar {\n"
"    border-top: 1px solid palette(midlight);\n"
"}\n"
"\n"
"QStatusBar::item {\n"
"    border: none;\n"
"}\n"
"")
        self.statusBar.setObjectName("statusBar")
        MainWindow.setStatusBar(self.statusBar)
        self.dockWidget = QtWidgets.QDockWidget(MainWindow)
        self.dockWidget.setStyleSheet("QDockWidget > QWidget {\n"
"    border-right: 1px solid palette(midlight);\n"
"}")
        self.dockWidget.setObjectName("dockWidget")
        self.dockWidgetContents = QtWidgets.QWidget()
        self.dockWidgetContents.setObjectName("dockWidgetContents")
        self.gridLayout = QtWidgets.QGridLayout(self.dockWidgetContents)
        self.gridLayout.setContentsMargins(0, 0, 1, 0)
        self.gridLayout.setObjectName("gridLayout")
        self.filterWidget = QtWidgets.QWidget(self.dockWidgetContents)
        self.filterWidget.setStyleSheet("QToolButton {\n"
"     padding: 1px;\n"
"     border: 1px solid transparent;\n"
"     border-radius: 2px;\n"
"     background: transparent;\n"
"    height: 14px;\n"
"}\n"
"\n"
"QToolButton:hover {\n"
"        border: 1px solid  rgba( 92, 163, 255, 20% );\n"
"        background-color: rgba( 92, 163, 255, 10% );\n"
"}\n"
"\n"
"QToolButton:checked, QToolButton:hover:pressed {\n"
"        border: 1px solid  rgba( 92, 163, 255, 50% );\n"
"        background-color: rgba( 92, 163, 255, 25% );\n"
"}")
        self.filterWidget.setObjectName("filterWidget")
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout(self.filterWidget)
        self.horizontalLayout_5.setContentsMargins(4, 5, 4, 0)
        self.horizontalLayout_5.setSpacing(4)
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.filterLineEdit = QtWidgets.QLineEdit(self.filterWidget)
        self.filterLineEdit.setStyleSheet("QLineEdit {\n"
"     border: 1px solid #ddd;\n"
"     border-radius: 2px;\n"
"     padding: 1px 5px 2px 5px;\n"
"     background: transparent;\n"
"    font-size: 11px;\n"
" }\n"
"")
        self.filterLineEdit.setClearButtonEnabled(True)
        self.filterLineEdit.setObjectName("filterLineEdit")
        self.horizontalLayout_5.addWidget(self.filterLineEdit)
        self.filterToolButton = QtWidgets.QToolButton(self.filterWidget)
        self.filterToolButton.setStyleSheet("\n"
"QToolButton {\n"
"    width: 20px;\n"
"    padding-right: 10px;\n"
"}\n"
"\n"
"QToolButton::menu-arrow {\n"
"    image: url(:/icons/down_arrow.png);\n"
"}\n"
"\n"
"QToolButton::menu-button {\n"
"    border: 1px transparent #76797C;\n"
"    border-top-right-radius: 6px;\n"
"    border-bottom-right-radius: 6px;\n"
"    width: 16px;\n"
"    outline: none;\n"
"}\n"
"")
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(":/icons/filter.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.filterToolButton.setIcon(icon5)
        self.filterToolButton.setCheckable(True)
        self.filterToolButton.setPopupMode(QtWidgets.QToolButton.MenuButtonPopup)
        self.filterToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonIconOnly)
        self.filterToolButton.setAutoRaise(True)
        self.filterToolButton.setObjectName("filterToolButton")
        self.horizontalLayout_5.addWidget(self.filterToolButton)
        self.tableViewToolButton = QtWidgets.QToolButton(self.filterWidget)
        self.tableViewToolButton.setStyleSheet("")
        icon6 = QtGui.QIcon()
        icon6.addPixmap(QtGui.QPixmap(":/icons/list-16x16.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.tableViewToolButton.setIcon(icon6)
        self.tableViewToolButton.setCheckable(True)
        self.tableViewToolButton.setAutoRaise(True)
        self.tableViewToolButton.setObjectName("tableViewToolButton")
        self.horizontalLayout_5.addWidget(self.tableViewToolButton)
        self.gridLayout.addWidget(self.filterWidget, 0, 0, 1, 1)
        self.variablesStackedWidget = QtWidgets.QStackedWidget(self.dockWidgetContents)
        self.variablesStackedWidget.setObjectName("variablesStackedWidget")
        self.treePage = QtWidgets.QWidget()
        self.treePage.setObjectName("treePage")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.treePage)
        self.gridLayout_5.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.treeView = QtWidgets.QTreeView(self.treePage)
        self.treeView.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.treeView.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.treeView.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
        self.treeView.setObjectName("treeView")
        self.gridLayout_5.addWidget(self.treeView, 0, 0, 1, 1)
        self.variablesStackedWidget.addWidget(self.treePage)
        self.tablePage = QtWidgets.QWidget()
        self.tablePage.setObjectName("tablePage")
        self.gridLayout_6 = QtWidgets.QGridLayout(self.tablePage)
        self.gridLayout_6.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.tableView = QtWidgets.QTreeView(self.tablePage)
        self.tableView.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.tableView.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.tableView.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
        self.tableView.setRootIsDecorated(False)
        self.tableView.setObjectName("tableView")
        self.gridLayout_6.addWidget(self.tableView, 0, 0, 1, 1)
        self.variablesStackedWidget.addWidget(self.tablePage)
        self.gridLayout.addWidget(self.variablesStackedWidget, 1, 0, 1, 1)
        self.dockWidget.setWidget(self.dockWidgetContents)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(1), self.dockWidget)
        self.toolBar = QtWidgets.QToolBar(MainWindow)
        self.toolBar.setMovable(False)
        self.toolBar.setFloatable(False)
        self.toolBar.setObjectName("toolBar")
        MainWindow.addToolBar(QtCore.Qt.TopToolBarArea, self.toolBar)
        self.actionSimulate = QtWidgets.QAction(MainWindow)
        icon7 = QtGui.QIcon()
        icon7.addPixmap(QtGui.QPixmap(":/icons/play.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionSimulate.setIcon(icon7)
        self.actionSimulate.setObjectName("actionSimulate")
        self.actionSettings = QtWidgets.QAction(MainWindow)
        self.actionSettings.setCheckable(True)
        icon8 = QtGui.QIcon()
        icon8.addPixmap(QtGui.QPixmap(":/icons/settings.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionSettings.setIcon(icon8)
        self.actionSettings.setObjectName("actionSettings")
        self.actionOpen = QtWidgets.QAction(MainWindow)
        icon9 = QtGui.QIcon()
        icon9.addPixmap(QtGui.QPixmap(":/icons/folder_open.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionOpen.setIcon(icon9)
        self.actionOpen.setObjectName("actionOpen")
        self.actionShowResults = QtWidgets.QAction(MainWindow)
        self.actionShowResults.setCheckable(True)
        icon10 = QtGui.QIcon()
        icon10.addPixmap(QtGui.QPixmap(":/icons/stats.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionShowResults.setIcon(icon10)
        self.actionShowResults.setObjectName("actionShowResults")
        self.actionNewWindow = QtWidgets.QAction(MainWindow)
        self.actionNewWindow.setObjectName("actionNewWindow")
        self.actionFilterInputs = QtWidgets.QAction(MainWindow)
        self.actionFilterInputs.setCheckable(True)
        self.actionFilterInputs.setChecked(True)
        self.actionFilterInputs.setObjectName("actionFilterInputs")
        self.actionFilterOutputs = QtWidgets.QAction(MainWindow)
        self.actionFilterOutputs.setCheckable(True)
        self.actionFilterOutputs.setChecked(True)
        self.actionFilterOutputs.setObjectName("actionFilterOutputs")
        self.actionFilterParameters = QtWidgets.QAction(MainWindow)
        self.actionFilterParameters.setCheckable(True)
        self.actionFilterParameters.setChecked(True)
        self.actionFilterParameters.setObjectName("actionFilterParameters")
        self.actionFilterLocalVariables = QtWidgets.QAction(MainWindow)
        self.actionFilterLocalVariables.setCheckable(True)
        self.actionFilterLocalVariables.setObjectName("actionFilterLocalVariables")
        self.actionFilterIndependentVariables = QtWidgets.QAction(MainWindow)
        self.actionFilterIndependentVariables.setCheckable(True)
        self.actionFilterIndependentVariables.setObjectName("actionFilterIndependentVariables")
        self.actionFilterCalculatedParameters = QtWidgets.QAction(MainWindow)
        self.actionFilterCalculatedParameters.setCheckable(True)
        self.actionFilterCalculatedParameters.setObjectName("actionFilterCalculatedParameters")
        self.actionOpenFMI2Spec = QtWidgets.QAction(MainWindow)
        self.actionOpenFMI2Spec.setObjectName("actionOpenFMI2Spec")
        self.actionOpenFMI1SpecCS = QtWidgets.QAction(MainWindow)
        self.actionOpenFMI1SpecCS.setObjectName("actionOpenFMI1SpecCS")
        self.actionOpenFMI1SpecME = QtWidgets.QAction(MainWindow)
        self.actionOpenFMI1SpecME.setObjectName("actionOpenFMI1SpecME")
        self.actionClose = QtWidgets.QAction(MainWindow)
        self.actionClose.setObjectName("actionClose")
        self.actionExit = QtWidgets.QAction(MainWindow)
        self.actionExit.setObjectName("actionExit")
        self.actionShowLog = QtWidgets.QAction(MainWindow)
        self.actionShowLog.setCheckable(True)
        icon11 = QtGui.QIcon()
        icon11.addPixmap(QtGui.QPixmap(":/icons/log.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionShowLog.setIcon(icon11)
        self.actionShowLog.setObjectName("actionShowLog")
        self.actionOpenTestFMUs = QtWidgets.QAction(MainWindow)
        self.actionOpenTestFMUs.setObjectName("actionOpenTestFMUs")
        self.actionSaveResult = QtWidgets.QAction(MainWindow)
        self.actionSaveResult.setObjectName("actionSaveResult")
        self.actionSavePlottedResult = QtWidgets.QAction(MainWindow)
        self.actionSavePlottedResult.setObjectName("actionSavePlottedResult")
        self.actionCreateDesktopShortcut = QtWidgets.QAction(MainWindow)
        self.actionCreateDesktopShortcut.setObjectName("actionCreateDesktopShortcut")
        self.actionShowAboutDialog = QtWidgets.QAction(MainWindow)
        self.actionShowAboutDialog.setObjectName("actionShowAboutDialog")
        self.actionOpenWebsite = QtWidgets.QAction(MainWindow)
        self.actionOpenWebsite.setObjectName("actionOpenWebsite")
        self.actionAddFileAssociation = QtWidgets.QAction(MainWindow)
        self.actionAddFileAssociation.setObjectName("actionAddFileAssociation")
        self.actionShowReleaseNotes = QtWidgets.QAction(MainWindow)
        self.actionShowReleaseNotes.setObjectName("actionShowReleaseNotes")
        self.actionReload = QtWidgets.QAction(MainWindow)
        icon12 = QtGui.QIcon()
        icon12.addPixmap(QtGui.QPixmap(":/icons/reload-16x16.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionReload.setIcon(icon12)
        self.actionReload.setObjectName("actionReload")
        self.actionSaveChanges = QtWidgets.QAction(MainWindow)
        self.actionSaveChanges.setObjectName("actionSaveChanges")
        self.actionLoadStartValues = QtWidgets.QAction(MainWindow)
        self.actionLoadStartValues.setObjectName("actionLoadStartValues")
        self.actionCreateCMakeProject = QtWidgets.QAction(MainWindow)
        self.actionCreateCMakeProject.setEnabled(False)
        self.actionCreateCMakeProject.setObjectName("actionCreateCMakeProject")
        self.actionCompilePlatformBinary = QtWidgets.QAction(MainWindow)
        self.actionCompilePlatformBinary.setEnabled(False)
        self.actionCompilePlatformBinary.setObjectName("actionCompilePlatformBinary")
        self.actionAddRemoting = QtWidgets.QAction(MainWindow)
        self.actionAddRemoting.setEnabled(False)
        self.actionAddRemoting.setObjectName("actionAddRemoting")
        self.actionAddCoSimulationWrapper = QtWidgets.QAction(MainWindow)
        self.actionAddCoSimulationWrapper.setEnabled(False)
        self.actionAddCoSimulationWrapper.setObjectName("actionAddCoSimulationWrapper")
        self.actionCreateJupyterNotebook = QtWidgets.QAction(MainWindow)
        self.actionCreateJupyterNotebook.setEnabled(False)
        self.actionCreateJupyterNotebook.setObjectName("actionCreateJupyterNotebook")
        self.actionValidateFMU = QtWidgets.QAction(MainWindow)
        self.actionValidateFMU.setEnabled(False)
        self.actionValidateFMU.setObjectName("actionValidateFMU")
        self.menu_File.addAction(self.actionOpen)
        self.menu_File.addAction(self.actionNewWindow)
        self.menu_File.addSeparator()
        self.menu_File.addAction(self.actionLoadStartValues)
        self.menu_File.addSeparator()
        self.menu_File.addAction(self.actionReload)
        self.menu_File.addSeparator()
        self.menu_File.addAction(self.actionSaveChanges)
        self.menu_File.addSeparator()
        self.menu_File.addAction(self.actionSaveResult)
        self.menu_File.addAction(self.actionSavePlottedResult)
        self.menu_File.addSeparator()
        self.menu_File.addAction(self.actionClose)
        self.menu_File.addSeparator()
        self.menu_File.addAction(self.actionExit)
        self.menuFMI_Specifications.addAction(self.actionOpenFMI2Spec)
        self.menuFMI_Specifications.addSeparator()
        self.menuFMI_Specifications.addAction(self.actionOpenFMI1SpecCS)
        self.menuFMI_Specifications.addAction(self.actionOpenFMI1SpecME)
        self.menu_Help.addAction(self.menuFMI_Specifications.menuAction())
        self.menu_Help.addAction(self.actionOpenTestFMUs)
        self.menu_Help.addSeparator()
        self.menu_Help.addAction(self.actionOpenWebsite)
        self.menu_Help.addAction(self.actionShowReleaseNotes)
        self.menu_Help.addSeparator()
        self.menu_Help.addAction(self.actionShowAboutDialog)
        self.menu_Tools.addAction(self.actionCreateDesktopShortcut)
        self.menu_Tools.addAction(self.actionAddFileAssociation)
        self.menu_Tools.addSeparator()
        self.menu_Tools.addAction(self.actionValidateFMU)
        self.menu_Tools.addAction(self.actionCompilePlatformBinary)
        self.menu_Tools.addAction(self.actionCreateJupyterNotebook)
        self.menu_Tools.addAction(self.actionCreateCMakeProject)
        self.menu_Tools.addAction(self.actionAddRemoting)
        self.menu_Tools.addAction(self.actionAddCoSimulationWrapper)
        self.menubar.addAction(self.menu_File.menuAction())
        self.menubar.addAction(self.menu_Tools.menuAction())
        self.menubar.addAction(self.menu_Help.menuAction())
        self.toolBar.addAction(self.actionOpen)
        self.toolBar.addAction(self.actionReload)
        self.toolBar.addSeparator()
        self.toolBar.addAction(self.actionSettings)
        self.toolBar.addAction(self.actionShowLog)
        self.toolBar.addAction(self.actionShowResults)
        self.toolBar.addSeparator()
        self.toolBar.addAction(self.actionSimulate)

        self.retranslateUi(MainWindow)
        self.stackedWidget.setCurrentIndex(0)
        self.solverComboBox.setCurrentIndex(1)
        self.variablesStackedWidget.setCurrentIndex(0)
        self.actionClose.triggered.connect(MainWindow.close)
        self.inputCheckBox.toggled['bool'].connect(self.inputFilenameLineEdit.setEnabled)
        self.inputCheckBox.toggled['bool'].connect(self.selectInputButton.setEnabled)
        self.outputIntervalRadioButton.toggled['bool'].connect(self.outputIntervalLineEdit.setEnabled)
        self.maxSamplesRadioButton.toggled['bool'].connect(self.maxSamplesLineEdit.setEnabled)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "FMPy"))
        self.label_11.setText(_translate("MainWindow", "Drop FMUs here or"))
        self.openButton.setText(_translate("MainWindow", "open an FMU"))
        self.recentFilesGroupBox.setTitle(_translate("MainWindow", "Recent Files"))
        self.infoGroupBox.setTitle(_translate("MainWindow", "Model Info"))
        self.label_3.setText(_translate("MainWindow", "FMI Version"))
        self.label_20.setText(_translate("MainWindow", "Variables"))
        self.label_8.setText(_translate("MainWindow", "Generation Date"))
        self.label_12.setText(_translate("MainWindow", "Event Indicators"))
        self.label_10.setText(_translate("MainWindow", "Continuous States"))
        self.label_4.setText(_translate("MainWindow", "Model Name"))
        self.label_5.setText(_translate("MainWindow", "FMI Type"))
        self.label.setText(_translate("MainWindow", "Platforms"))
        self.label_22.setText(_translate("MainWindow", "Description"))
        self.label_7.setText(_translate("MainWindow", "Generation Tool"))
        self.settingsGroupBox.setTitle(_translate("MainWindow", "Simulation"))
        self.label_14.setText(_translate("MainWindow", "Relative Tolerance"))
        self.applyDefaultStartValuesCheckBox.setText(_translate("MainWindow", "Apply default start values"))
        self.inputCheckBox.setText(_translate("MainWindow", "Input"))
        self.label_2.setText(_translate("MainWindow", "Step Size"))
        self.selectInputButton.setToolTip(_translate("MainWindow", "Select input file"))
        self.selectInputButton.setText(_translate("MainWindow", "..."))
        self.logFMICallsCheckBox.setText(_translate("MainWindow", "Log FMI calls"))
        self.debugLoggingCheckBox.setText(_translate("MainWindow", "Debug Logging"))
        self.label_13.setText(_translate("MainWindow", "Solver"))
        self.inputFilenameLineEdit.setPlaceholderText(_translate("MainWindow", "No input file selected"))
        self.stepSizeLineEdit.setText(_translate("MainWindow", "1e-3"))
        self.solverComboBox.setItemText(0, _translate("MainWindow", "Fixed-step"))
        self.solverComboBox.setItemText(1, _translate("MainWindow", "Variable-step"))
        self.relativeToleranceLineEdit.setText(_translate("MainWindow", "1e-5"))
        self.maxSamplesLineEdit.setText(_translate("MainWindow", "500"))
        self.outputIntervalRadioButton.setText(_translate("MainWindow", "Output Interval"))
        self.outputIntervalLineEdit.setText(_translate("MainWindow", "1e-2"))
        self.maxSamplesRadioButton.setText(_translate("MainWindow", "Max. Samples"))
        self.portsGroupBox.setTitle(_translate("MainWindow", "Ports"))
        self.clearLogButton.setToolTip(_translate("MainWindow", "Clear messages"))
        self.clearLogButton.setText(_translate("MainWindow", "clear"))
        self.clearLogOnStartButton.setToolTip(_translate("MainWindow", "Clear messages on simulation start"))
        self.clearLogOnStartButton.setText(_translate("MainWindow", "clear on start"))
        self.logFilterLineEdit.setPlaceholderText(_translate("MainWindow", "filter messages"))
        self.showDebugMessagesButton.setToolTip(_translate("MainWindow", "Show debug messages"))
        self.showDebugMessagesButton.setText(_translate("MainWindow", "0"))
        self.showInfoMessagesButton.setToolTip(_translate("MainWindow", "Show info messages"))
        self.showInfoMessagesButton.setText(_translate("MainWindow", "0"))
        self.showWarningMessagesButton.setToolTip(_translate("MainWindow", "Show warning messages"))
        self.showWarningMessagesButton.setText(_translate("MainWindow", "0"))
        self.showErrorMessagesButton.setToolTip(_translate("MainWindow", "Show error messages"))
        self.showErrorMessagesButton.setText(_translate("MainWindow", "0"))
        self.menu_File.setTitle(_translate("MainWindow", "&File"))
        self.menu_Help.setTitle(_translate("MainWindow", "&Help"))
        self.menuFMI_Specifications.setTitle(_translate("MainWindow", "FMI &Specifications"))
        self.menu_Tools.setTitle(_translate("MainWindow", "&Tools"))
        self.filterLineEdit.setPlaceholderText(_translate("MainWindow", "filter variables"))
        self.filterToolButton.setToolTip(_translate("MainWindow", "Filter variables"))
        self.filterToolButton.setText(_translate("MainWindow", "..."))
        self.tableViewToolButton.setToolTip(_translate("MainWindow", "Toggle list view"))
        self.toolBar.setWindowTitle(_translate("MainWindow", "toolBar"))
        self.actionSimulate.setText(_translate("MainWindow", "Simulate"))
        self.actionSimulate.setToolTip(_translate("MainWindow", "Start simulation"))
        self.actionSettings.setText(_translate("MainWindow", "Settings"))
        self.actionSettings.setToolTip(_translate("MainWindow", "Show settings"))
        self.actionOpen.setText(_translate("MainWindow", "&Open"))
        self.actionOpen.setToolTip(_translate("MainWindow", "Open file"))
        self.actionOpen.setShortcut(_translate("MainWindow", "Ctrl+O"))
        self.actionShowResults.setText(_translate("MainWindow", "Results"))
        self.actionShowResults.setToolTip(_translate("MainWindow", "Show results"))
        self.actionNewWindow.setText(_translate("MainWindow", "&New Window"))
        self.actionNewWindow.setShortcut(_translate("MainWindow", "Ctrl+Shift+N"))
        self.actionFilterInputs.setText(_translate("MainWindow", "Inputs"))
        self.actionFilterInputs.setToolTip(_translate("MainWindow", "Filter input variables"))
        self.actionFilterOutputs.setText(_translate("MainWindow", "Outputs"))
        self.actionFilterOutputs.setToolTip(_translate("MainWindow", "Filter output variables"))
        self.actionFilterParameters.setText(_translate("MainWindow", "Parameters"))
        self.actionFilterParameters.setToolTip(_translate("MainWindow", "Filter parameters"))
        self.actionFilterLocalVariables.setText(_translate("MainWindow", "Local"))
        self.actionFilterLocalVariables.setToolTip(_translate("MainWindow", "Filter local variables"))
        self.actionFilterIndependentVariables.setText(_translate("MainWindow", "Independent"))
        self.actionFilterIndependentVariables.setToolTip(_translate("MainWindow", "Filter independent variables"))
        self.actionFilterCalculatedParameters.setText(_translate("MainWindow", "Calculated Parameters"))
        self.actionFilterCalculatedParameters.setToolTip(_translate("MainWindow", "Filter calculated parameters"))
        self.actionOpenFMI2Spec.setText(_translate("MainWindow", "FMI 2.0.2"))
        self.actionOpenFMI2Spec.setToolTip(_translate("MainWindow", "FMI 2.0.2"))
        self.actionOpenFMI1SpecCS.setText(_translate("MainWindow", "FMI 1.0.1 Co-Simulation"))
        self.actionOpenFMI1SpecME.setText(_translate("MainWindow", "FMI 1.0.1 for Model Exchange"))
        self.actionClose.setText(_translate("MainWindow", "&Close"))
        self.actionClose.setToolTip(_translate("MainWindow", "Close the current window"))
        self.actionClose.setShortcut(_translate("MainWindow", "Ctrl+W"))
        self.actionExit.setText(_translate("MainWindow", "E&xit"))
        self.actionExit.setToolTip(_translate("MainWindow", "Close all windows"))
        self.actionShowLog.setText(_translate("MainWindow", "Log"))
        self.actionShowLog.setToolTip(_translate("MainWindow", "Show log"))
        self.actionOpenTestFMUs.setText(_translate("MainWindow", "&Test FMUs"))
        self.actionOpenTestFMUs.setToolTip(_translate("MainWindow", "Test FMUs on fmi-standard.org"))
        self.actionSaveResult.setText(_translate("MainWindow", "&Save Result..."))
        self.actionSaveResult.setShortcut(_translate("MainWindow", "Ctrl+S"))
        self.actionSavePlottedResult.setText(_translate("MainWindow", "Save &Plotted Result..."))
        self.actionSavePlottedResult.setToolTip(_translate("MainWindow", "Save plotted result"))
        self.actionCreateDesktopShortcut.setText(_translate("MainWindow", "Create Desktop &Shortcut"))
        self.actionShowAboutDialog.setText(_translate("MainWindow", "&About FMPy"))
        self.actionOpenWebsite.setText(_translate("MainWindow", "FMPy &Website"))
        self.actionAddFileAssociation.setText(_translate("MainWindow", "Associate with *.fmu Files"))
        self.actionAddFileAssociation.setToolTip(_translate("MainWindow", "Add file association for *.fmu"))
        self.actionShowReleaseNotes.setText(_translate("MainWindow", "&Release Notes"))
        self.actionReload.setText(_translate("MainWindow", "&Reload"))
        self.actionReload.setShortcut(_translate("MainWindow", "F5"))
        self.actionSaveChanges.setText(_translate("MainWindow", "Save Changes"))
        self.actionLoadStartValues.setText(_translate("MainWindow", "Load Start Values"))
        self.actionCreateCMakeProject.setText(_translate("MainWindow", "Create C&Make Project..."))
        self.actionCompilePlatformBinary.setText(_translate("MainWindow", "Compile Platform &Binary"))
        self.actionAddRemoting.setText(_translate("MainWindow", "Add 32-bit &Remoting"))
        self.actionAddCoSimulationWrapper.setText(_translate("MainWindow", "Add Co-Simulation &Wrapper"))
        self.actionCreateJupyterNotebook.setText(_translate("MainWindow", "Create &Jupyter Notebook..."))
        self.actionValidateFMU.setText(_translate("MainWindow", "&Validate FMU"))
from pyqtgraph import GraphicsLayoutWidget
from . import icons_rc
